package net.pingex.dcf;

import net.pingex.dcf.commands.CommandRegistry;
import net.pingex.dcf.commands.InternalCommands;
import net.pingex.dcf.core.Configuration;
import net.pingex.dcf.core.GatewayConnectionManager;
import net.pingex.dcf.modularity.PluginLoader;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import sx.blah.discord.api.ClientBuilder;

/**
 * Main class for DCF
 */
public class DiscordCommandableFramework
{
	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(DiscordCommandableFramework.class);

	/**
	 * Main entry point
	 */
	public static void main(String[] args)
	{
		// Load configuration
		Configuration.load();
		Configuration.init();

		LOGGER.info("Hello World");
		LOGGER.info(Configuration.BOT_NAME);
		LOGGER.info("Framework version {}", DiscordCommandableFramework.class.getPackage().getImplementationVersion() != null ? DiscordCommandableFramework.class.getPackage().getImplementationVersion() : "UNKNOWN");

		// Load plugins
		PluginLoader.getInstance().discoverPluginsDirectory();
		PluginLoader.getInstance().bulkLoadPlugins();

		// Set up initial connection
		ClientBuilder builder = new ClientBuilder();
		if(Configuration.isConnectionToken())	builder.withToken(Configuration.CONNECTION_TOKEN);
		else									builder.withLogin(Configuration.CONNECTION_USERNAME, Configuration.CONNECTION_PASSWORD);
		GatewayConnectionManager.registerConnection(builder);

		// Register internal commands
		new InternalCommands().getCommands().forEach(CommandRegistry::registerCommand);

		// Run plugins
		PluginLoader.getInstance().bulkRunPlugins();
	}
}
