package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Context;
import net.pingex.dcf.commands.options.ICommandOption;
import net.pingex.dcf.core.Configuration;
import java.util.function.Predicate;

/**
 * This class defines the default behavior of a command when checking for permissions.
 */
public class DefaultPermissionOption implements ICommandOption
{
	private Value defaultValue;

	public DefaultPermissionOption(Value defaultValue)
	{
		this.defaultValue = defaultValue;
	}

	public Value getDefaultValue()
	{
		return defaultValue;
	}

	@Override
	public String getOptionName()
	{
		return "Default permission";
	}

	@Override
	public String getOptionDescription()
	{
		return "This option defines the default behavior of a command when checking for permissions.";
	}

	/**
	 * Default behavior when a permissions provider doesn't return any value
	 */
	public enum Value implements Predicate<Context>
	{
		/**
		 * Everyone is allowed to run the command.
		 */
		EVERYONE(context -> true),

		/**
		 * Only the guild owner is allowed to run the command.
		 */
		GUILD_OWNER(context -> !context.getChannel().isPrivate() && context.getUser().getID().equals(context.getChannel().getGuild().getOwnerID())),

		/**
		 * Only the bot owner is allowed to run the command.
		 */
		BOT_OWNER(context -> context.getUser().getID().equals(Configuration.BOT_OWNER)),

		/**
		 * Guild Owner x Bot Owner
		 */
		ANY_OWNER(context -> GUILD_OWNER.test(context) || BOT_OWNER.test(context)),

		/**
		 * Nobody is allowed to run the command
		 */
		NONE(context -> false);

		Value(Predicate<Context> predicate)
		{
			this.predicate = predicate;
		}

		private Predicate<Context> predicate;

		@Override
		public boolean test(Context o)
		{
			return predicate.test(o);
		}
	}
}
