package net.pingex.dcf.permissions.audit;

import net.pingex.dcf.commands.Context;
import net.pingex.dcf.commands.audit.AuditResult;
import net.pingex.dcf.commands.audit.IAuditComponentProvider;
import net.pingex.dcf.commands.options.ICommandOption;
import net.pingex.dcf.permissions.DefaultPermissionOption;
import java.util.Optional;

/**
 * This class checks permission against a default behavior defined in a command.
 */
public class DefaultPermissionCheck implements IAuditComponentProvider
{
	@Override
	public AuditResult doAudit(Context context)
	{
		Optional<ICommandOption> option = context.getCommand().getOptions().stream().filter(i -> i instanceof DefaultPermissionOption).findFirst();

		if(!option.isPresent())
			return new AuditResult(AuditResult.ResultCode.NOOP, "No default permission defined.");

		if(((DefaultPermissionOption) option.get()).getDefaultValue().test(context))
			return new AuditResult(AuditResult.ResultCode.PASS);
		else
			return new AuditResult(AuditResult.ResultCode.FAIL, "Default policy denied access to command.");
	}

	@Override
	public String name()
	{
		return "Default permission check";
	}

	@Override
	public String description()
	{
		return "Checks against a default permission option defined in a command.";
	}

	@Override
	public int priority()
	{
		return 5;
	}
}
