package net.pingex.dcf.permissions.audit;

import net.pingex.dcf.commands.Context;
import net.pingex.dcf.commands.audit.AuditResult;
import net.pingex.dcf.commands.audit.IAuditComponentProvider;
import net.pingex.dcf.permissions.ICommandPermissionsProvider;
import sx.blah.discord.handle.obj.IRole;

/**
 * This class checks permission against a rule defined for a specific role in a guild.
 */
public class UserGuildRoleCheck implements IAuditComponentProvider
{
	/**
	 * Permissions provider to use
	 */
	private ICommandPermissionsProvider provider;

	/**
	 * Provider for this object to use
	 * @param provider Permissions provider this object will use
	 */
	public UserGuildRoleCheck(ICommandPermissionsProvider provider)
	{
		this.provider = provider;
	}

	@Override
	public AuditResult doAudit(Context context)
	{
		// Check for guild
		if(context.getChannel().getGuild() == null)
			return new AuditResult(AuditResult.ResultCode.NOOP, "This channel is not part of a guild.");

		if(context.getChannel().getGuild().getRolesForUser(context.getUser()) != null) // User has roles
			for(IRole i : context.getChannel().getGuild().getRolesForUser(context.getUser()))
			{
				Boolean returnedQuery = provider.validateGroup(i, context.getCommand());
				if(returnedQuery != null)
				{
					if(returnedQuery)	return new AuditResult(AuditResult.ResultCode.PASS);
					else				return new AuditResult(AuditResult.ResultCode.FAIL, "Role rule denied access for this user.");
				}
			}
		else // User has no role
			return new AuditResult(AuditResult.ResultCode.NOOP, "No role defined for this user.");

		// No rule for any role
		return new AuditResult(AuditResult.ResultCode.NOOP, "No rule defined for any role this user has.");
	}

	@Override
	public String name()
	{
		return "Guild role check";
	}

	@Override
	public String description()
	{
		return "Checks permission against a rule defined for a specific role in a guild.";
	}

	@Override
	public int priority()
	{
		return 3;
	}
}
